/*
 * Diskheap FREE defines...
 *
 * Copyright 2002 by Gray Watson
 *
 * This file is part of the diskheap package.
 *
 * Permission to use, copy, modify, and distribute this software for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of Gray Watson not be used in advertising
 * or publicity pertaining to distribution of the document or software
 * without specific, written prior permission.
 *
 * Gray Watson makes no representations about the suitability of the
 * software described herein for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * The author may be contacted via http://256.com/gray/
 *
 * $Id: diskheap_free.h,v 1.14 2002/03/29 06:01:15 gray Exp $
 */

#ifndef __DISKHEAP_FREE_H__
#define __DISKHEAP_FREE_H__

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

/*
 * This structure holds free-slot information in a skip-list format.
 * Skip lists have variable number of next pointers so the fs_next_p
 * pointer array may actually have more elements depending on how
 * large an allocation we make.  Don't you love these C hacks.
 */
typedef struct skip_slot_st {
  unsigned int		fs_block_num;	/* block number of free space */
  unsigned int		fs_offset;	/* offset in block of free space */
  /* size of free space in blocks if offset is 0 otherwise bytes */
  unsigned int		fs_size;
  /*
   * Array of next pointers.  This may extend past the end of the
   * function if we allocate for space larger than the structure.
   */
  struct skip_slot_st	*fs_next_p[1];
} skip_slot_t;

/*
 * This macro helps us determine how much memory we need to store to
 * hold all of the next pointers in the skip-list entry.
 */
#define SKIP_SLOT_SIZE(next_n)	\
	(sizeof(skip_slot_t) + sizeof(struct skip_slot_st *) * ((next_n) - 1))

/*<<<<<<<<<<  The below prototypes are auto-generated by fillproto */

/*
 * int _dh_free_startup
 *
 * DESCRIPTION:
 *
 * Initialize the free slot structures in a new diskheap.
 *
 * RETURNS:
 *
 * Success - DISKHEAP_ERROR_NONE
 *
 * Failure - Diskheap error code
 *
 * ARGUMENTS:
 *
 * dh_p <-> Disk head we are working with.
 */
extern
int	_dh_free_startup(diskheap_t *dh_p);

/*
 * int _dh_free_shutdown
 *
 * DESCRIPTION:
 *
 * Shutdown any structures in the free list.
 *
 * RETURNS:
 *
 * None.
 *
 * ARGUMENTS:
 *
 * dh_p <-> Disk head we are working with.
 */
extern
void	_dh_free_shutdown(diskheap_t *dh_p);

/*
 * int _dh_free_add
 *
 * DESCRIPTION:
 *
 * Add a free entry to our administrative structures.
 *
 * RETURNS:
 *
 * Success - DISKHEAP_ERROR_NONE
 *
 * Failure - Diskheap error code
 *
 * ARGUMENTS:
 *
 * dh_p <-> Disk head we are working with.
 *
 * block_num -> Block-number of the free space.
 *
 * offset -> Offset we are searching for.  This can be 0 if we are
 * searching for a size but don't care about where.
 *
 * size_block_n -> Size that we are searching for blocks.  Either this
 * or size_byte_n will be 0.
 *
 * size_byte_n -> Size that we are searching for in bytes.  Either
 * this or size_block_n will be 0.
 */
extern
int	_dh_free_add(diskheap_t *dh_p, const unsigned int block_num,
		     const unsigned int offset,
		     const unsigned int size_block_n,
		     const unsigned int size_byte_n);

/*
 * int _dh_free_get
 *
 * DESCRIPTION:
 *
 * Find in the free list the best matching space to fit a certain
 * block/size chunk.
 *
 * RETURNS:
 *
 * Success - DISKHEAP_ERROR_NONE
 *
 * Failure - DISKHEAP_ERROR_NO_FREE is no [appropriate] free space is
 * available or another error code
 *
 * ARGUMENTS:
 *
 * dh_p <-> Disk pointer to modify.
 *
 * block_n -> Number of free blocks we need for our allocation.  May
 * be 0 if we want a certain number of bytes.
 *
 * byte_n -> Number of free blocks we need for our allocation.  May be
 * 0 if if want a certain number of blocks.
 *
 * block_num_p <- Pointer to an unsigned integer which, if not NULL,
 * will be set to the block-number of the free block to use.
 *
 * offset_p <- Pointer to an unsigned integer which, if not NULL, will
 * be set to the offset of the free space to use.
 *
 * found_block_np <- Pointer to an unsigned int which be set to the
 * number of blocks that we found in the free list.
 *
 * found_byte_np <- Pointer to an unsigned int which be set to the
 * number of bytes that we found in the free list.
 */
extern
int	_dh_free_get(diskheap_t *dh_p, const unsigned int block_n,
		     const unsigned int byte_n,
		     unsigned int *block_num_p,
		     unsigned int *offset_p,
		     unsigned int *found_block_np,
		     unsigned int *found_byte_np);

/*<<<<<<<<<<   This is end of the auto-generated output from fillproto. */

#ifdef __cplusplus
}
#endif


#endif /* ! __DISKHEAP_FREE_H__ */
